--- api.py.bak	2007-09-17 17:18:36.000000000 -0400
+++ api.py	2007-09-17 19:11:47.000000000 -0400
@@ -24,6 +24,10 @@
 from trac.util.text import shorten_line
 from trac.wiki import IWikiSyntaxProvider, Formatter
 
+try:
+    import threading
+except ImportError:
+    import dummy_threading as threading
 
 class ITicketChangeListener(Interface):
     """Extension point interface for components that require notification when
@@ -68,6 +72,10 @@
         [TracTickets#Assign-toasDrop-DownList Assign-to as Drop-Down List]
         (''since 0.9'').""")
 
+    cached_users = None
+    cached_perm_id = None
+    _cache_lock = threading.Lock()
+
     # Public API
 
     def get_available_actions(self, ticket, perm_):
@@ -99,13 +107,43 @@
         field = {'name': 'owner', 'label': 'Owner'}
         if self.restrict_owner:
             field['type'] = 'select'
-            users = []
-            perm = PermissionSystem(self.env)
-            for username, name, email in self.env.get_known_users(db):
-                if perm.get_user_permissions(username).get('TICKET_MODIFY'):
-                    users.append(username)
-            field['options'] = users
-            field['optional'] = True
+
+            perm_id = None
+            try:
+                db = self.env.get_db_cnx()
+                cursor = db.cursor()
+                cursor.execute("SELECT value FROM system WHERE name = 'perm_id'")
+                perm_row = cursor.fetchone()
+                cursor.close()
+                if not perm_row:
+                    raise TracError('perm_id KVP in system table missing',
+                            'Invalid restricted owner caching data - is the trigger running?')
+                perm_id = perm_row[0];
+            except:
+               pass
+
+            """We check once before locking and once after to prevent locking when we don't need to """
+            if self.cached_users is None or (perm_id is not None and perm_id > self.cached_perm_id):
+ 
+                self._cache_lock.acquire()
+                try:
+                    if self.cached_users is None or (perm_id is not None and perm_id > self.cached_perm_id):
+                        users = []
+                        perm = PermissionSystem(self.env)
+                        for username, name, email in self.env.get_known_users(db):
+                            if perm.get_user_permissions(username).get('TICKET_MODIFY'):
+                                users.append(username)
+                        self.cached_users = users;
+                        if perm_id is not None:
+                            self.cached_perm_id = perm_id
+                finally:
+                    self._cache_lock.release()
+
+            if self.cached_users is not None:
+                field['options'] = self.cached_users
+                field['optional'] = True
+            else:
+                field['type'] = 'text'
         else:
             field['type'] = 'text'
         fields.append(field)
